@@ -7,7 +7,7 @@
 import pandas as pd
 from numpy.random import random as randn
 import time
-import warnings
+from statsmodels.api import tsa
 
 # An unofficial google trends API
 from pytrends.request import TrendReq
@@ -46,7 +46,7 @@ class Trendex:
         so collapsing into larger time-frames is done by averages ex-post.
 
     gen_index: Binary, optional
-        If true, then go ahead and instantiate class to generate indices.
+        If true, then go ahead and instantiate class to generate index.
         Default is True.
 
     plot: boolean, optional
@@ -67,10 +67,23 @@ class Trendex:
         Currently defaults to random intervals of mean 5 or 7 seconds depending on
         where in the code. Remove this at your own peril (Google lockout).
 
+    seasonal_adjust: boolean, optional (default = True)
+        If True, then seasonally adjust the series (recommended). Seasonally
+        adjusted trends are always constructed and saved in trends_sa,
+        but this incorporates them into the index that is automatically constructed.
+
+    sa_method: str, optional (default = 'trends')
+        Three methods are acceptable: 'trends' or 'index' or 'both'.
+        The trends method seasonally adjusts each trend separately (before adding).
+        The index method seasonally adjusts only the final series.
+        The both method seasonally adjusts separately and the final index.
+
+
+
 
     Returns (back to class instance)
     -------
-    self.indices: Dataframe
+    self.gti: Series
         This is the normalized indexes made from the underlying data. It is
         the main thing returned from this function.
 
@@ -79,6 +92,9 @@ class Trendex:
         You could use this to plot individual keywords in the index.
         Will differ from raw_trends_adjusted if frequency is changed from daily.
 
+    self.trends_sa: Dataframe
+        It is the trends dataframe where each trend has been seasonally adjusted.
+
     self.raw_trends_adjusted: Dictionary
         These are the adjusted (using overlapping timeframes) raw results
         for each term. Index of dictionary corresponds to index of timechunks.
@@ -106,16 +122,20 @@ class Trendex:
     overlap = 30 # An arbitrary length of time for the series to overlap
     kw_limit = 20 # Default is to break kw_list into chunks with "+" operator
 
-    def __init__(self, kw_list, geo, lang, date_start=None, date_end=None, frequency='daily',
-        gen_index=True, plot=True, kw_list_split=True, benchmark_select=True, slowdown=True):
+    def __init__(self, kw_list, geo, lang, date_start=None, date_end=None,
+                 frequency='daily', gen_index=True, plot=True,
+                 kw_list_split=True, benchmark_select=True, slowdown=True,
+                 seasonal_adjust=True, sa_method='trends'):
 
         # Input Arguments
-        self.user_kw_list = kw_list
+        self.user_kw_list = kw_list.copy()
         self.geo = geo
         self.lang = lang
         self.user_date_start = date_start
         self.user_date_end = date_end
         self.frequency = frequency
+        self.sa_method = sa_method
+        self.seasonal = seasonal_adjust
         self.slowdown = slowdown
         self.benchmark_select = benchmark_select
 
@@ -124,8 +144,9 @@ class Trendex:
             self.kw_list = self.combine_kw_list(kw_list)
         else:
             self.kw_list = kw_list
-        self.benchmark, self.search_groups = self.get_benchmark()
         self.date_start, self.date_end = self.auto_dates()
+        self.benchmark, self.search_groups = self.get_benchmark()
+
         self.timechunks = self.get_timechunks()
 
         # Initialized None Arguments For make_index
@@ -134,7 +155,8 @@ class Trendex:
         self.raw_trends = None
         self.raw_trends_adjusted = None
         self.trends = None
-        self.indices = None
+        self.trends_sa = None
+        self.gti = None
 
 
         # Make the Index, unless False:
@@ -144,14 +166,14 @@ class Trendex:
     def __repr__(self) -> str:
 
         # Return info on how to generate indices
-        if self.indices is None:
+        if self.gti is None:
             result = 'An instantiation of WBTrends Class. '\
                 'Run make_index() to get result'
             return result
 
 
         # Otherwise return a string representation for indices attribute.
-        df = self.indices
+        df = self.gti
         buf = StringIO("")
         if df._info_repr():
             df.info(buf=buf)
@@ -176,7 +198,7 @@ class Trendex:
             show_dimensions=show_dimensions,
         )
 
-        return '\n'.join(['The Calculated Indices (stored in object.indices):'
+        return '\n'.join(['The Calculated Index (stored in object.gti):'
                           ,buf.getvalue()])
 
     def make_index(self,plot=True):
@@ -190,14 +212,14 @@ class Trendex:
             You must instantiate the class in a valid way to run this function.
 
         plot: Binary, optional
-            If you put True then will plot indices. The default is True.
+            If you put True then will plot index. The default is True.
 
         Returns (back to class instance)
         -------
-        self.indices: Dataframe
+        self.gti: Dataframe
             This is the normalized indexes made from the underlying data. It is
             the main thing returned from this function.
-            It also is returned to self.indices.
+            It also is returned to self.gti.
 
         self.trends: Dataframe
             It is the adjusted and combined series for each term searched.
@@ -278,17 +300,33 @@ class Trendex:
 
         # Save also the collapsed trend series
         self.trends = trends
+        trends_sa = trends.apply(self.sadjust,axis=0)
+        self.trends_sa = trends_sa
 
-        indices = pd.concat([
-            trends.apply(self.normalize).sum(axis=1).to_frame('GTI (Normalized)'),
-            trends.sum(axis=1).to_frame('GTI (Standard)')
-            ],axis=1).apply(self.normalize)
+
+        # Old way
+        # indices = pd.concat([
+        #         trends_sa.apply(self.normalize).sum(axis=1).to_frame('GTI (Normalized)'),
+        #         trends_sa.sum(axis=1).to_frame('GTI (Standard)')
+        #         ],axis=1).apply(self.normalize)
+        if self.seasonal and self.sa_method == 'trends':
+            indices = self.normalize(trends_sa.sum(axis=1))
+            indices.name = 'GTI'
+        elif self.seasonal and self.sa_method == 'index':
+            indices = self.normalize(self.sadjust(trends.sum(axis=1)))
+            indices.name = 'GTI'
+        elif self.season and self.sa_method == 'both':
+            indices = self.normalize(self.sadjust(trends_sa.sum(axis=1)))
+            indices.name = 'GTI'
+        else:
+            indices = self.normalize(trends.sum(axis=1))
+            indices.name = 'GTI'
 
         if plot:
             indices.plot()
 
         # The final thing is to add the indices
-        self.indices = indices
+        self.gti = indices
 
         return self
 
@@ -357,9 +395,8 @@ class Trendex:
                         break
                 else:
                     if self.too_small(df[self.benchmark]):
-                        warnings.warn('Benchmark term %s is optimatlly selected, '\
-                        'but performs poorly between %s and %s, '\
-                        'will automatically adjust' %(self.benchmark,date_start,date_end))
+                        print('Benchmark term %s is optimally selected, but performs '\
+                            'poorly between %s and %s' %(self.benchmark,date_start,date_end))
 
                 # Get rid of partial days
                 df = df.loc[df.isPartial.astype('str').eq('False'),ss]
@@ -384,12 +421,12 @@ class Trendex:
 
     def get_benchmark(self):
         # Limit on google trends searches is 5 words else need benchmark term
-        if len(self.kw_list) > 5 and not benchmark_select:
+        if len(self.kw_list) > 5 and not self.benchmark_select:
             benchmark = self.kw_list[0]
             search_groups = list(self.chunks(self.kw_list))
-        elif len(self.kw_list) > 5 and benchmark_select:
+        elif len(self.kw_list) > 5 and self.benchmark_select:
             benchmark = self.optimal_benchmark()
-            words = self.kw_list
+            words = self.kw_list.copy()
             # put optimal benchmark first here
             words.insert(0, words.pop(words.index(benchmark)))
             search_groups = list(self.chunks(words))
@@ -522,6 +559,13 @@ class Trendex:
         else:
             return False
 
+    @staticmethod
+    def sadjust(x):
+        """ Accepts a series with datetime index, returns statsmodels trend+resid """
+        y = tsa.seasonal_decompose(x,extrapolate_trend='freq')
+        z = y.resid + y.trend
+        return z
+
     @staticmethod
     def normalize(x):
         return (x-x.mean())/x.std()
